/*
 * vdr-podcatcher - A plugin for the Linux Video Disk Recorder
 * Copyright (c) 2006 Tobias Grimm <vdr@e-tobi.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: circularpespacketbuffer.cc 3253 2006-04-23 13:35:28Z svntobi $
 *
 */

#include "circularpespacketbuffer.h"
#include "pesaudiopacket.h"

using namespace std;

CircularPesPacketBuffer::CircularPesPacketBuffer(unsigned int capacity)
:capacity(capacity+2), eof(false)
{
    buffer = new ref_ptr<PesAudioPacket>[this->capacity];
    readPtr = buffer;
    writePtr = buffer + 1;
}

CircularPesPacketBuffer::~CircularPesPacketBuffer()
{
    delete[] buffer;
}

ref_ptr<PesAudioPacket> CircularPesPacketBuffer::GetPesPacket()
{
    ref_ptr<PesAudioPacket>* newReadPointer = NextPointerLocation(readPtr,
      writePtr);
    if (newReadPointer)
    {
        readPtr = newReadPointer;
        return *readPtr;
    }
    else
    {
        return ref_ptr<PesAudioPacket>();
    }
}


bool CircularPesPacketBuffer::PutPesPacket(ref_ptr<PesAudioPacket> packet)
{
    ref_ptr<PesAudioPacket>* newWritePointer = NextPointerLocation(writePtr,
      readPtr);

    if (newWritePointer)
    {
        *writePtr = packet;
        writePtr = newWritePointer;
        return true;
    }
    else
    {
        return false;
    }
}

bool CircularPesPacketBuffer::IsEmpty()
{
    if (writePtr == (readPtr + 1))
    {
        return true;
    }
    else
    {
        return false;
    }

    return true;
}

ref_ptr<PesAudioPacket>* CircularPesPacketBuffer::NextPointerLocation(
  ref_ptr<PesAudioPacket>* currentLocation, ref_ptr<PesAudioPacket>* limit)
{
    currentLocation++;

    if (currentLocation >= (buffer + capacity))
    {
        currentLocation = buffer;
    }

    if (currentLocation == limit)
    {
        return 0;
    }
    else
    {
        return currentLocation;
    }
}

void CircularPesPacketBuffer::SetEof(bool eof)
{
    this->eof = eof;
}

bool CircularPesPacketBuffer::Eof()
{
    return eof;
}
