/*
 * vdr-podcatcher - A plugin for the Linux Video Disk Recorder
 * Copyright (c) 2006 Tobias Grimm <vdr@e-tobi.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: curldownloader.cc 3211 2006-04-14 21:01:37Z svntobi $
 *
 */

#include <sstream>
#include <curl/curl.h>
#include "curldownloader.h"
#include "downloadcache.h"
#include "download.h"

using namespace std;

CurlDownloader::CurlDownloader(IDownloadCache& downloadCache)
:downloadCache(downloadCache)
{
}

bool CurlDownloader::PerformDownload(Download& download)
{
    stringstream stream;
    CURL *curl;
    CURLcode res;


    curl = curl_easy_init();
    if(curl)
    {
        curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION,
          &CurlDownloader::WriteDataToStream);
        curl_easy_setopt(curl, CURLOPT_WRITEDATA, &stream);
        curl_easy_setopt(curl, CURLOPT_PROGRESSFUNCTION,
          &CurlDownloader::ProgressCallback);
        curl_easy_setopt(curl, CURLOPT_PROGRESSDATA, &download);
        curl_easy_setopt (curl, CURLOPT_NOPROGRESS, 0);
//
// TODO: Maybe make this somehow configurable by the user or encapsulate
//       in debug-ifdef
//
//        curl_easy_setopt(curl, CURLOPT_VERBOSE, 1);
//
//
        curl_easy_setopt(curl, CURLOPT_FOLLOWLOCATION, 1);
        curl_easy_setopt(curl, CURLOPT_URL, download.GetUrl().c_str());
        res = curl_easy_perform(curl);
        curl_easy_cleanup(curl);
        if (CURLE_OK == res)
        {
            downloadCache.Put(stream, download.GetUrl());
            return true;
        }
    }
    return false;
}

size_t CurlDownloader::WriteDataToStream(void *buffer, size_t sizeOfMemblock,
  size_t numberOfMemblocks, void *streamPointer)
{
    ostream* stream = (ostream*) streamPointer;

    try
    {
        int bufferSize = sizeOfMemblock * numberOfMemblocks;
        stream->write((const char *) buffer, bufferSize);
        return bufferSize;
    }
    catch(...)
    {
    }

    return 0;
}

int CurlDownloader::ProgressCallback(void *downloadPtr, double downloadSize,
  double downloaded, double uploadSize, double uploaded)
{
    Download* download = (Download*)downloadPtr;
    download->SetSize(int(downloadSize));
    download->SetDownloaded(int(downloaded));
    return 0;
}
