/*
 * vdr-podcatcher - A plugin for the Linux Video Disk Recorder
 * Copyright (c) 2006 Tobias Grimm <vdr@e-tobi.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: feedmenupresenter.cc 2511 2006-01-28 14:05:09Z svntobi $
 *
 */

#include "feedmenupresenter.h"
#include "servicelocator.h"
#include "downloadpool.h"
#include "feedparser.h"
#include "feedsources.h"
#include "listmenu.h"

FeedMenuPresenter::FeedMenuPresenter(IServiceLocator& serviceLocator)
:serviceLocator(serviceLocator)
{
}

void FeedMenuPresenter::Initialize(IListMenu* listMenu)
{
    menu = listMenu;

    menu->SetTitle(tr("Select Feed"));
    menu->SetRedHelp(tr("Update"));

    serviceLocator.GetFeedSources()->GetFeeds(feeds);

    for(FeedList::iterator i = feeds.begin(); i != feeds.end(); i++)
    {
        if (serviceLocator.GetFeedParser()->Parse(*i))
        {
            menu->AddItem(i->GetTitle());
        }
        else
        {
            menu->AddItem("??\?");
        }
    }
}

eOSState FeedMenuPresenter::ProcessKey(int selectedFeed, eKeys key,
  eOSState state)
{
    if (!state == osUnknown)
    {
        return state;
    }

    switch (key)
    {
        case kRed:
        {
            ref_ptr<Download> download(
              new Download(feeds[selectedFeed].GetUrl()));
            serviceLocator.GetDownloadPool()->AddDownload(download);
            return osContinue;
        }

        case kOk:
        {
            cOsdMenu* itemMenu = serviceLocator.CreateItemMenu(feeds[selectedFeed]);
            menu->ShowSubMenu(itemMenu);
            return osContinue;
        }

        default:
        {
            return state;
        }
    }
}
