/*
 * vdr-podcatcher - A plugin for the Linux Video Disk Recorder
 * Copyright (c) 2006 Tobias Grimm <vdr@e-tobi.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: itembuttonsupdater.cc 3211 2006-04-14 21:01:37Z svntobi $
 *
 */

#include "itembuttonsupdater.h"
#include "menu.h"
#include "servicelocator.h"
#include "downloadcache.h"
#include "refptr.h"
#include "download.h"
#include "downloadpool.h"
#include <sstream>

using namespace std;

ItemButtonsUpdater::ItemButtonsUpdater(IServiceLocator& serviceLocator)
:serviceLocator(serviceLocator)
{
}

void ItemButtonsUpdater::UpdateButtonHelp(IMenu* menu, Item item)
{
    UpdateRedHelp(menu, item);
    UpdateGreenHelp(menu, item);
}

void ItemButtonsUpdater::UpdateGreenHelp(IMenu* menu, Item item)
{
    menu->SetGreenHelp("");

    if (item.IsPodcast())
    {
        if (serviceLocator.GetDownloadCache()->IsCached(item.GetMp3Url()))
        {
            menu->SetGreenHelp(tr("Play"));
        }
    }
}

void ItemButtonsUpdater::UpdateRedHelp(IMenu* menu, Item item)
{
    if (!item.IsPodcast())
    {
        menu->SetRedHelp("");
        return;
    }

    ref_ptr<Download> download =
      serviceLocator.GetDownloadPool()->GetDownloadByUrl(item.GetMp3Url());

    if (download.get())
    {
        if (download->GetSize() > 0)
        {
            int percent = 100 * download->GetDownloaded() / download->GetSize();
            stringstream s;
            s << "> " << percent << "%" << " <";
            menu->SetRedHelp(s.str());
        }
        else
        {
            menu->SetRedHelp(tr(">queued<"));
        }
    }
    else
    {
        menu->SetRedHelp(tr("Download"));
    }
}
