/*
 * vdr-podcatcher - A plugin for the Linux Video Disk Recorder
 * Copyright (c) 2006 Tobias Grimm <vdr@e-tobi.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: itemviewpresenter.cc 2518 2006-01-29 23:33:10Z svntobi $
 *
 */

#include "itembuttonsupdater.h"
#include "itemviewpresenter.h"
#include <vdr/i18n.h>
#include "itemview.h"
#include "servicelocator.h"
#include "downloadcache.h"
#include "downloadpool.h"
#include <vdr/player.h>

using namespace std;

ItemViewPresenter::ItemViewPresenter(IServiceLocator& serviceLocator,
  string feedTitle, Item item)
:serviceLocator(serviceLocator), feedTitle(feedTitle), item(item),
  itemButtonsUpdater(serviceLocator)
{
}

void ItemViewPresenter::Initialize(IItemView* view)
{
    this->view = view;
    view->SetTitle(feedTitle);

    view->SetDescription(item.GetDate().ToLongString() + "\n\n" +
      item.GetTitle() + "\n\n" + item.GetDescription());

    itemButtonsUpdater.UpdateButtonHelp(view, item);
}

eOSState ItemViewPresenter::ProcessKey(eOSState state, eKeys key)
{
    itemButtonsUpdater.UpdateButtonHelp(view, item);

    switch (key)
    {
        case kRed:
        case kGreen:
        case kNone:
        {
            // indicates the parent, that it can process these keys
            return osUnknown;
        }

        default:
        {
            return state;
        }
    }
}
