/*
 * vdr-podcatcher - A plugin for the Linux Video Disk Recorder
 * Copyright (c) 2006 Tobias Grimm <vdr@e-tobi.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: mpaframeheader.h 3211 2006-04-14 21:01:37Z svntobi $
 *
 */

#ifndef ___MPAFRAMEHEADER_H
#define ___MPAFRAMEHEADER_H

#define MPA_HEADER_SIZE 4

typedef unsigned char MpaHeaderData[4];

class MpaFrameHeader
{
    private:
        static const int mpegVersions[4];
        static const int layers[4];
        static const int samplingRates[2][4];
        static const int bitRates[2][3][16];
        static const int samplesPerFrame[2][3];
        int mpegVersion;
        int layer;
        int samplingRate;
        int bitRate;
        int channelMode;
        int padding;

    private:
        unsigned int GetValueOfBitsWithinByteBoundary(
          unsigned char buffer[MPA_HEADER_SIZE], int indexOfFirstBit,
          int numberOfBits);
        int ExtractMpegVersion(MpaHeaderData headerData);
        int ExtractLayer(MpaHeaderData headerData);
        int ExtractBitRate(MpaHeaderData headerData);
        int ExtractSamplingRate(MpaHeaderData headerData);
        int ExtractPadding(MpaHeaderData headerData);
        int ExtractChannelMode(MpaHeaderData headerData);
        int GetSamplesPerFrame();
        int GetSlotSize();

    public:
        MpaFrameHeader(MpaHeaderData headerData);
        unsigned int GetSamplingRate();
        unsigned int GetBitRate();
        unsigned int GetSideInformationSize();
        int GetFrameSize();
};

#endif
