/*
 * vdr-podcatcher - A plugin for the Linux Video Disk Recorder
 * Copyright (c) 2006 Tobias Grimm <vdr@e-tobi.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: osdlistmenu.cc 3219 2006-04-16 15:06:35Z svntobi $
 *
 */

#include "osdlistmenu.h"
#include "listmenupresenter.h"

using namespace std;

OsdListMenu::OsdListMenu(ref_ptr<IListMenuPresenter> presenter, int tabStop1)
:cOsdMenu("", tabStop1), presenter(presenter)
{
    this->presenter->Initialize(this);
}

void OsdListMenu::SetTitle(string title)
{
    cOsdMenu::SetTitle(title.c_str());
}

eOSState OsdListMenu::ProcessKey(eKeys Key)
{
    eOSState state = cOsdMenu::ProcessKey(Key);

    return presenter->ProcessKey(Current(), Key, state);
}

void OsdListMenu::AddItem(std::string item)
{
    Add(new cOsdItem(item.c_str()));
}

eOSState OsdListMenu::ShowSubMenu(cOsdMenu* menu)
{
    return AddSubMenu(menu);
}

void OsdListMenu::Clear()
{
    cOsdMenu::Clear();
}

void OsdListMenu::Refresh()
{
    cOsdMenu::Display();
}

void OsdListMenu::SetRedHelp(string text)
{
    redHelp = text;
    UpdateHelpButtons();
}

void OsdListMenu::SetGreenHelp(string text)
{
    greenHelp = text;
    UpdateHelpButtons();
}

void OsdListMenu::SetYellowHelp(string text)
{
    yellowHelp = text;
    UpdateHelpButtons();
}

void OsdListMenu::SetBlueHelp(string text)
{
    blueHelp = text;
    UpdateHelpButtons();
}

void OsdListMenu::UpdateHelpButtons()
{
    cOsdMenu::SetHelp(EmptyToNull(redHelp.c_str()),
      EmptyToNull(greenHelp.c_str()), EmptyToNull(yellowHelp.c_str()),
      EmptyToNull(blueHelp.c_str()));
}

const char* OsdListMenu::EmptyToNull(const char * str)
{
    if (*str)
    {
        return str;
    }
    else
    {
        return NULL;
    }
}

bool OsdListMenu::IsActiveMenu()
{
    return !HasSubMenu();
}
