/*
 * vdr-podcatcher - A plugin for the Linux Video Disk Recorder
 * Copyright (c) 2006 Tobias Grimm <vdr@e-tobi.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: podcatcherplugin.h 3219 2006-04-16 15:06:35Z svntobi $
 *
 */

#ifndef ___PODCATCHERPLUGIN_H
#define ___PODCATCHERPLUGIN_H

#include <vdr/plugin.h>
#include "configuration.h"
#include "servicelocator.h"
#include "thread.h"

class PodcatcherPlugin: public cPlugin, public IConfiguration
{
    private:
        std::string cacheDirName;
        std::string sourcesFileName;
        int maxCacheAge;
        ref_ptr<IServiceLocator> serviceLocator;
        ref_ptr<IThread> downloadThread;

    private:
        bool CacheDirIsAccessible();

    public:
        PodcatcherPlugin();
        void SetServiceLocator(ref_ptr<IServiceLocator> serviceLocator);
        // <cPlugin>
        const char* Version();
        const char* Description();
        const char* MainMenuEntry();
        bool Initialize();
        bool Start();
        void Housekeeping();
        const char* CommandLineHelp();
        bool ProcessArgs(int argc, char* argv[]);
        cOsdObject* MainMenuAction();
        cMenuSetupPage* SetupMenu();
        bool SetupParse(const char *Name, const char *Value);
        // </cPlugin>
        // <IConfiguration>
        std::string GetCacheDirName();
        std::string GetSourcesFileName();
        int GetMaxCacheAge();
        void SetMaxCacheAge(int age);
        // </IConfiguration>
};

extern "C" void* VDRPluginCreator();

#endif
