/*
 * vdr-podcatcher - A plugin for the Linux Video Disk Recorder
 * Copyright (c) 2006 Tobias Grimm <vdr@e-tobi.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: refptr.h 3226 2006-04-16 23:08:51Z svntobi $
 *
 */

#ifndef ___REFPTR__H
#define ___REFPTR__H

template <class X>
class ref_ptr
{
    private:
        X* managedPointer;
        int* refCounter;

    public:
        explicit ref_ptr(X* ptr = 0)
        {
            ManagePointer(ptr);
        }

        ref_ptr(const ref_ptr& refPtr)
        {
            AssignRefPtr(refPtr);
        }

        ~ref_ptr()
        {
            ReleasePointer();
        }

        X* get() const
        {
            return managedPointer;
        }

        X* operator->()
        {
            return managedPointer;
        }

        X& operator*()
        {
            return *managedPointer;
        }

        ref_ptr& operator=(const ref_ptr& refPtr)
        {
            if (this != &refPtr)
            {
                ReleasePointer();

                AssignRefPtr(refPtr);
            }

            return *this;
        }

        ref_ptr operator=(X* ptr)
        {
            if (this->managedPointer != ptr)
            {
                ReleasePointer();
                ManagePointer(ptr);
            }

            return *this;
        }

    private:
        void ReleasePointer()
        {
            if (*refCounter > 0)
            {
                (*refCounter)--;
            }
            else
            {
                delete managedPointer;
                delete refCounter;
            }
        }

        void AssignRefPtr(const ref_ptr& refPtr)
        {
            refCounter = refPtr.refCounter;
            (*refCounter)++;
            managedPointer = refPtr.get();
        }

        void ManagePointer(X* ptr)
        {
            this->managedPointer = ptr;
            refCounter = new int(0);
        }
};

#endif
