/*
 * vdr-podcatcher - A plugin for the Linux Video Disk Recorder
 * Copyright (c) 2006 Tobias Grimm <vdr@e-tobi.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: servicelocatorimpl.h 3219 2006-04-16 15:06:35Z svntobi $
 *
 */

#ifndef ___SERVICELOCATORIMPL_H
#define ___SERVICELOCATORIMPL_H

#include "servicelocator.h"
#include "refptr.h"

class IDownloader;
class IPesPacketBuffer;
class IConfiguration;

class ServiceLocatorImpl: public IServiceLocator
{
    private:
        IConfiguration& configuration;
        ref_ptr<IFeedSources> feedSources;
        ref_ptr<IFeedParser> feedParser;
        ref_ptr<IDownloadPool> downloadPool;
        ref_ptr<IDownloadCache> downloadCache;
        ref_ptr<IThread> downloadThread;
        ref_ptr<IDownloader> downloader;
        ref_ptr<IFeedUpdater> feedUpdater;
        ref_ptr<IErrorLogger> errorLogger;

    private:
        IDownloader* GetDownloader();
        ref_ptr<IPesPacketBuffer> CreateSynchedPcmBuffer();

    public:
        ServiceLocatorImpl(IConfiguration& configuration);
        // <IServiceLocator>
        IFeedSources* GetFeedSources();
        IFeedParser* GetFeedParser();
        IDownloadPool* GetDownloadPool();
        IDownloadCache* GetDownloadCache();
        IThread* GetDownloadThread();
        IFeedUpdater* GetFeedUpdater();
        ref_ptr<IThreadAction> CreateDownloadAction();
        cOsdMenu* CreateFeedMenu();
        cOsdMenu* CreateItemMenu(Feed feed);
        cOsdMenu* CreateItemView(Feed feed, Item item);
        cControl* CreatePlayerControl(Item item);
        IErrorLogger* GetErrorLogger();
        cMenuSetupPage* CreateSetupMenu();
        // </IServiceLocator>
};

#endif
