/*
 * vdr-podcatcher - A plugin for the Linux Video Disk Recorder
 * Copyright (c) 2006 Tobias Grimm <vdr@e-tobi.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: syncheddownloadpool.cc 3219 2006-04-16 15:06:35Z svntobi $
 *
 */

#include "syncheddownloadpool.h"
#include "sleeper.h"

using namespace std;

SynchedDownloadPool::SynchedDownloadPool(ref_ptr<IDownloadPool> downloadPool,
  ref_ptr<ISleeper> sleeper)
:downloadPool(downloadPool), sleeper(sleeper)
{
}

void SynchedDownloadPool::AddDownload(ref_ptr<Download> download)
{
    downloadPool->AddDownload(download);
    sleeper->WakeUp();
}

void SynchedDownloadPool::RemoveDownload(ref_ptr<Download> download)
{
    downloadPool->RemoveDownload(download);
}

ref_ptr <Download> SynchedDownloadPool::GetNextDownload()
{
    ref_ptr<Download> download = downloadPool->GetNextDownload();

    if (!download.get())
    {
        sleeper->Sleep();
    }

    return download;
}

ref_ptr <Download> SynchedDownloadPool::GetDownloadByUrl(string url)
{
    return downloadPool->GetDownloadByUrl(url);
}
