/*
 * vdr-podcatcher - A plugin for the Linux Video Disk Recorder
 * Copyright (c) 2006 Tobias Grimm <vdr@e-tobi.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: synchedpespacketbuffer.cc 3253 2006-04-23 13:35:28Z svntobi $
 *
 */

#include "synchedpespacketbuffer.h"
#include "pesaudiopacket.h"
#include "sleeper.h"

using namespace std;

SynchedPesPacketBuffer::SynchedPesPacketBuffer(ref_ptr<IPesPacketBuffer> wrappedBuffer,
  ref_ptr<ISleeper> getSleeper, ref_ptr<ISleeper> putSleeper)
:wrappedBuffer(wrappedBuffer), getSleeper(getSleeper), putSleeper(putSleeper)
{
}

ref_ptr<PesAudioPacket> SynchedPesPacketBuffer::GetPesPacket()
{
    ref_ptr<PesAudioPacket> packet = wrappedBuffer->GetPesPacket();

    if (packet.get())
    {
        return packet;
    }
    else
    {
        getSleeper->Sleep();
        return wrappedBuffer->GetPesPacket();
    }
}

bool SynchedPesPacketBuffer::PutPesPacket(ref_ptr<PesAudioPacket> packet)
{
    if (wrappedBuffer->PutPesPacket(packet))
    {
        return true;
    }
    else
    {
        putSleeper->Sleep();
        return wrappedBuffer->PutPesPacket(packet);
    }

}

bool SynchedPesPacketBuffer::IsEmpty()
{
    return wrappedBuffer->IsEmpty();
}

void SynchedPesPacketBuffer::SetEof(bool eof)
{
    wrappedBuffer->SetEof(eof);
}

bool SynchedPesPacketBuffer::Eof()
{
    return wrappedBuffer->Eof();
}
