/*
 * vdr-podcatcher - A plugin for the Linux Video Disk Recorder
 * Copyright (c) 2006 Tobias Grimm <vdr@e-tobi.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: threadsafedownloadpool.cc 3219 2006-04-16 15:06:35Z svntobi $
 *
 */

#include "threadsafedownloadpool.h"
#include "thread.h"
#include "sleeper.h"

using namespace std;

ThreadSafeDownloadPool::ThreadSafeDownloadPool(ref_ptr<IDownloadPool> downloadPool,
  ref_ptr<ICriticalSection> criticalSection)
:downloadPool(downloadPool), criticalSection(criticalSection)
{
}

void ThreadSafeDownloadPool::AddDownload(ref_ptr<Download> download)
{
    criticalSection->Enter();
    downloadPool->AddDownload(download);
    criticalSection->Leave();
}

void ThreadSafeDownloadPool::RemoveDownload(ref_ptr<Download> download)
{
    criticalSection->Enter();
    downloadPool->RemoveDownload(download);
    criticalSection->Leave();
}

ref_ptr<Download> ThreadSafeDownloadPool::GetNextDownload()
{
    criticalSection->Enter();
    ref_ptr<Download> nextDownload = downloadPool->GetNextDownload();
    criticalSection->Leave();
    return nextDownload;
}

ref_ptr<Download> ThreadSafeDownloadPool::GetDownloadByUrl(string url)
{
    criticalSection->Enter();
    ref_ptr<Download> download = downloadPool->GetDownloadByUrl(url);
    criticalSection->Leave();
    return download;
}
