/*
 * vdr-podcatcher - A plugin for the Linux Video Disk Recorder
 * Copyright (c) 2006 Tobias Grimm <vdr@e-tobi.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: vdrpesplayer.cc 3342 2006-04-24 21:30:53Z svntobi $
 *
 */

#include "vdrpesplayer.h"
#include "thread.h"
#include "pesaudiopacket.h"
#include "servicelocator.h"
#include "pcmdatasource.h"

using namespace std;

VdrPesPlayer::VdrPesPlayer()
:cPlayer(pmAudioOnly), timeIndex(0)
{
}

VdrPesPlayer::~VdrPesPlayer()
{
    Detach();
}

void VdrPesPlayer::Init(ref_ptr<IThread> decodeThread,
  ref_ptr<IThread> playbackThread)
{
    this->decodeThread = decodeThread;
    this->playbackThread = playbackThread;
}

void VdrPesPlayer::Activate(bool On)
{
    if (On)
    {
        if (decodeThread.get() && playbackThread.get())
        {
            decodeThread->Start();
            playbackThread->Start();
        }
    }
    else
    {
        if (playbackThread.get())
        {
            playbackThread->Stop();
        }

        if (decodeThread.get())
        {
            decodeThread->Stop();
        }
    }
}

void VdrPesPlayer::PlayPes(PesAudioPacket& packet)
{
    cPoller poller;

    timeIndex = packet.GetTimeIndex();
    totalTime = packet.GetTotalTime();

    cPlayer::PlayPes(packet.GetBufferPtr(), packet.GetBufferSize());

    while (!DevicePoll(poller, 500))
    {
    }
}

unsigned int VdrPesPlayer::GetTimeIndex()
{
    return timeIndex;
}

int VdrPesPlayer::GetTotalTime()
{
    return totalTime;
}
